/**
#---------------------------------------------------------------------#
#                   	Mega-Chess V1.0 			      #
#                              					      #
#                  A simple tiny portable Mega Chess Game             #
#								      #
#                                                                     #
# (c)opyright 2001 all rights reserved                                #
#							              #	
# Authors: Open Source community with A.Costa et al                   #
#                                                                     #
#                                                                     #
# For info and support visit:                                         #
#								      #
# http://sourceforge.net/p/mega-chess 				      #            
#                  						      #
# Or								      #
#								      #
# http://www.seesolutions.it/mega-chess 			      #
#---------------------------------------------------------------------#
#
#

LICENSE

To be distributed with following text and gpl.txt file only:
Append your own license

GNU GPL (see gpl.txt)

Unless required by applicable law or agreed to in writing, software distributed
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
OF ANY KIND, either express or implied. See the License for the specific language
governing permissions and limitations under the License.

DISCLAIMER

THIS SOFTWARE AVAILABLE ON THE SITE www.seesolutions.it IS PROVIDED
"AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL SEE Solutions OR ANY OF THEIR CONTRIBUTORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

Without limiting the foregoing, SEE Solutions make no warranty that:

- the software will meet your requirements.
- the software will be uninterrupted, timely, secure or error-free.
- the results that may be obtained from the use of the software will be effective, accurate or reliable.
- the quality of the software will meet your expectations.
- any errors in the software obtained from the SEE Solutions web site will be corrected.
- Software and its documentation made available on the SEE Solutions web site:

could include technical or other mistakes, inaccuracies or typographical errors.
SEE contributors may make changes to the software or documentation made available
on its web site. may be out of date and SEE Solutions and its contributors make no
commitment to update such materials.

SEE Solutions and its contributors assume no responsibility for errors or ommissions
in the software or documentation available from the SEE Solutions web site.

In no event shall SEE Solutions, or it's contributors be liable to you or any third parties
for any special, punitive, incidental, indirect or consequential damages of any kind,
or any damages whatsoever, including, without limitation, those resulting from loss of
use, data or profits, whether or not SEE Solutions, or its contributors has been advised
of the possibility of such damages, and on any theory of liability, arising out of
or in connection with the use of this software.

The use of the software downloaded through the SEE Solutions site is done at your own
discretion and risk and with agreement that you will be solely responsible for any
amage to your computer system or loss of data that results from such activities.
No advice or information, whether oral or written, obtained by you from SEE Solutions,
its its website or its contributors shall create any warranty for the software.


For more information contact:  info@seesolutions.it
*/

/**
*This class contains all methods needed to do the various figure-move-checks, all useful methods
*(check for empty squares etc.), some rules such as InCheck, Mated, the method to perform the
*actual move and the counter for the 50-move-rule
*
*
*@version Mega-Chess V1.1
*@authors Armin Costa, Ivan Granruaz, Stefan Pizzinini, Davide Schuen
*
*/
import java.awt.*;

public class ChessMoves {
 /* ChessMoves class  */
   /**Constructor of the main class */
   MegaChess main;
   /**rows, and columns   */
   static int rows, columns;
   /** variable holding the the beginning status (white or black)  */
   static int beginning;
   /**keeps track of the number of moves   */
   static int moveNumber;
   /**contains the figure mappings*/
    static int[][] board;
   /** Which player moved last : 0 - white, 1 - black  */
   static int lastplayermoved;
   /**Constructor of the class Move */
   Move lastmove;
   /**shows last moves + STATUs   */
   TextField ta;
   /** shows 50-move-rule status    */
   TextField log;

/**
*gets the variables from the main class MegaChess and constructs Move()
*@param hetboard
*/
ChessMoves (MegaChess hetboard)
  {
    board   = hetboard.board;    // Copy pointer, not board itself
    rows    = hetboard.rows;
    columns = hetboard.columns;
    beginning = hetboard.beginning;
    moveNumber = hetboard.moveNumber;
    ta = hetboard.ta;
    log = hetboard.log;

        // Sets the booleans whether one can castle.
        // As white starts, that would correspond to black playing last
    lastplayermoved  = 1;
    lastmove = new Move();
}


/**
*copies the variables
*@param tocopy
*/
ChessMoves (ChessMoves tocopy)
{
    /**As white starts, that would correspond to black playing last*/
    lastplayermoved  = tocopy.lastplayermoved;
    lastmove = new Move ( tocopy.lastmove.startcol, tocopy.lastmove.startrow,
                          tocopy.lastmove.endcol,   tocopy.lastmove.endrow);
    /**Initiate current board according to parent board*/
    board = new int[columns][rows];
    for (int i = 0; i< columns; i++)
      for (int j=0; j< rows; j++)
       board[i][j]=tocopy.board[i][j];
}

/**
*initializes the current board according to the parent board
*/
public void SetBoard (BoardCopy tocopy)
{
    /**As white starts, that would correspond to black playing last*/
    lastplayermoved  = tocopy.lastplayermoved;
    lastmove = new Move ( tocopy.lastmove.startcol, tocopy.lastmove.startrow,
                          tocopy.lastmove.endcol,   tocopy.lastmove.endrow);
     /** Initiate current board according to parent board   */
    for (int i = 0; i< columns; i++)
      for (int j=0; j< rows; j++)
       board[i][j]=tocopy.board[i][j];
}

//////////////////////////////////////////////////////////////////
//  Legal-move-Check

/**
*Legalmove checks if on the end position there is a piece of the same color
*or the move is not semilegal
*in that case the move is not legal(false)
*variables: k1=startcolumn, r1=startrow, k2=endcolumn, r2=endrow
*@param k1,r1 give the startcoordinates
*@param k2,r2 give the endcoordinates
*/
  public boolean LegalMove (int k1, int r1, int k2, int r2){
    if(!SemiLegalMove(k1, r1, k2, r2)){
      return false;
    }
    if(TakingPieceOfSameColor(k1,r1, k2, r2)){   //checks if the coordinates ( k2, r2) correspond
      return false;                              // correspond to the same color and return false if true;
    }
   return true;
  }


/**
*A move is SemiLegal if it is legal for that figure without looking for check
*if the endposition is the same as the startpositon the move is not semilegal
*/
  public boolean SemiLegalMove(int k1, int r1, int k2, int r2){
    if (k1 == k2 && r1 == r2){
      return false;
    }
    switch (board[k1][r1])
    {
      case 0:            return false;
      case 1:            return LegalWhiteKingMove(k1,r1,k2,r2);
      case 2:            return LegalBlackKingMove(k1,r1,k2,r2);
      case 3:  case 4:   return LegalQueenMove(k1, r1, k2, r2);
      case 5:  case 6:   return LegalRookMove(k1,r1,k2,r2);
      case 7:  case 8:   return LegalKnightMove(k1,r1,k2,r2);
      case 9:  case 10:  return LegalBishopMove (k1,r1,k2,r2);
      case 11:           return LegalWhitePawnMove (k1,r1,k2,r2);
      case 12:           return LegalBlackPawnMove (k1,r1,k2,r2);

    }
    return true;
  }
///////////////////////////////////////////////////////////////////
//   Figure Moves


/**
*this method allows the white king to move in every direction only for 1 position
*the taking variable checks, if the end position is free(false) or not(true)
*/
  boolean LegalWhiteKingMove(int k1, int r1, int k2, int r2){
        boolean taking = (board[k2][r2] == 0 ) ? false : true;
        if (((r2 == r1 + 1)||(r2 == r1 - 1)) &&  ((k2 == k1 +1)||(k2 == k1 -1) ))
            return true;
        else if (((r1 == r2)&& ((k2 == k1 + 1)||(k2 == k1 - 1))))
            return true;
        else if (((k1 == k2) && ((r2 == r1 + 1)||(r2 == r1 -1))))
            return true;
        return false;

  }


 /**
 *allows the black king to move in every direction only for 1 position
 *the taking variable checks, if the end position is free(false), or not(true)
 */
  boolean LegalBlackKingMove(int k1, int r1, int k2, int r2)
    {
        boolean taking = (board[k2][r2] == 0 ) ? false : true;
        if (((r2 == r1 + 1)||(r2 == r1 - 1)) &&  ((k2 == k1 +1)||(k2 == k1 -1) ))
            return true;
        else if (((r1 == r2)&& ((k2 == k1 + 1)||(k2 == k1 - 1))))
            return true;
        else if (((k1 == k2) && ((r2 == r1 + 1)||(r2 == r1 -1))))
            return true;
        return false;
  }


  /**
 *allows the queen to move in every direction diagonal and straight and looks
 *if the positions inbetween start-end position are free
 *the taking variable checks, if the end position is free(false), or not(true)
 */
  boolean LegalQueenMove (int k1, int r1, int k2, int r2){
        boolean taking = (board[k2][r2] == 0 ) ? false : true;
        for (int h = 1; h < 8; h++){
            if ((r2 == r1 + h && (k2 == k1 + h  || k2 == k1-h)) ||
                (r2 == r1-h && (k2 == k1 + h || k2 == k1-h))){
                if ((IsSpaceFree1_xy(k1, r1, k2, r2) && (IsSpaceFree2_xy(k1, r1, k2, r2))) == true){
                    return true;
                }
            }
            else if((( r2 == r1 + h) && (k1 == k2 )) ||(( r2 == r1 - h) && (k1 == k2 ))||
                    ((r1 == r2) && (k2 == k1 + h))||((r1 == r2) && (k2 == k1 - h))){
                if ((IsSpaceFree1_xy(k1, r1, k2, r2) && (IsSpaceFree2_xy(k1, r1, k2, r2))) == true){
                    return true;
                }
            }
        }
        return false;
  }



  /**
 *allows the white pawn to move only straight forward the first time for 2 position
 *and all the other times for 1 position
 *the pawn can also take a figure, if it move for 1 position in the diagonal
 *the taking variable checks, if the end position is free(false), or not(true)
 */
 boolean LegalWhitePawnMove (int k1, int r1, int k2, int r2){
   boolean taking = (board[k2][r2] == 0 ) ? false : true;
   if (taking == true && r2 == r1 + 1 && ( (k1 == k2 +1) || (k1 == k2 -1)))
        return true;
   else if (taking == false && (r2 == r1 + 1) && k1 == k2)
        return true;
   else if (taking == false && r1 == 1 && r2 == 3 && k1 == k2 &&
            board[k1][2] == 0 )
        return true; // white double pawn step
   return false;
 }



 /**
 *allows the black pawn to move only straight forward the first time for 2 position
 *and all the other times for 1 position
 *the pawn can also take a figure, if it move for 1 position in the diagonal
 *the taking variable checks, if the end position is free(false), or not(true)
 */
 boolean LegalBlackPawnMove (int k1, int r1, int k2, int r2){
  boolean taking = (board[k2][r2] == 0 ) ? false : true;
  if (taking == true && r2 == r1 - 1 && ( (k1 == k2 +1) || (k1 == k2 -1)))
        return true;
   else if (taking == false && r2 == r1 - 1 && k1 == k2)
        return true;
   else if (taking == false && r1 == rows-2 && r2 == r1-2 && k1 == k2 &&
            board[k1][r1-1] == 0 )
        return true;  // black double pawn step
   return false;
 }


 /**
 *allows the knight to move for 2 positions straight forward or backward and 1 left or right or
 *to move 1 position forward or backward and 2 left or right
 *it does not take care if a figure is inbetween
 *the taking variable checks, if the end position is free(false), or not(true)
 */
 boolean LegalKnightMove(int k1, int r1, int k2, int r2){
   boolean taking = (board[k2][r2] == 0 ) ? false : true;

   if(taking == true && r2 == r1 +1 && ( (k1 == k2 +2) || (k1 == k2 -2)))
         return true;

   if(taking == true && r2 == r1 +2 && ( (k1 == k2 +1) || (k1 == k2 -1)))
         return true;

   if(taking == true && r2 == r1 -1 && ( (k1 == k2 +2) || (k1 == k2 -2)))
        return true;

   if(taking == true && r2 == r1 -2 && ( (k1 == k2 +1) || (k1 == k2 -1)))
        return true;

   if(taking == false && r2 == r1 +1 && ( (k1 == k2 +2) || (k1 == k2 -2)))
        return true;

   if(taking == false && r2 == r1 +2 && ( (k1 == k2 +1) || (k1 == k2 -1)))
        return true;

   if(taking == false && r2 == r1 -1 && ( (k1 == k2 +2) || (k1 == k2 -2)))
        return true;

   if(taking == false && r2 == r1 -2 && ( (k1 == k2 +1) || (k1 == k2 -1)))
        return true;

   return false;
 }



 /**
 *allows the bishop to move in a diagonal direction and calls a method which looks
 *if the positions inbetween start-end position are free
 *the taking variable checks, if the end position is free(false), or not(true)
 */
 boolean LegalBishopMove (int k1, int r1, int k2, int r2){

        boolean taking = (board[k2][r2] == 0 ) ? false : true;
        for (int h = 1; h < 8; h++){
            if ((r2 == r1 + h && (k2 == k1 + h  || k2 == k1-h)) ||
                (r2 == r1-h && (k2 == k1 + h || k2 == k1-h))){
                if (IsSpaceFree2_xy(k1, r1, k2, r2) == true){
                    return true;
                }
            }
        }
        return false;
 }


/**
*allows the rook to move in every straight direction and calls a method which looks
*if the positions inbetween start-end position are free
*the taking variable checks, if the end position is free(false), or not(true)
*/
 boolean LegalRookMove (int k1, int r1, int k2, int r2){
        //boolean legalmove = false;
        boolean taking = (board[k2][r2] == 0 ) ? false : true;
        for (int h = 1; h < 8; h++){
            if ((( r2 == r1 + h) && (k1 == k2 )) ||(( r2 == r1 - h) && (k1 == k2 ))||
                ((r1 == r2) && (k2 == k1 + h))||((r1 == r2) && (k2 == k1 - h))){
                if (IsSpaceFree1_xy(k1, r1, k2, r2) == true){
                    return true;
                }
            }
        }
        return false;

 }

/////////////////////////////////////////////////////////////////////////////////////////////
//  Some methods used to do various checks
/**
* The PieceOwer method returns 0 if white owns the piece. Otherwise it
* returns 1-.
*/
 public int PieceOwner(int k, int r){
   return 1- (board[k][r] %2) ; // 0 if white owns the piece,1-if black
 }


/**
* The RightPlayer method returns the player who has to do the move.
*/
 public boolean RightPlayer(int k1, int r1){
   return (PieceOwner(k1,r1) != lastplayermoved);
 }


/**
* The EmptySquare method returns true if a specified space of the board is
* empty. Otherwise it returns false.
*/
 public boolean EmptySquare(int col, int row){
    return (board[col][row] == 0) ? true : false;
 }


/**
* The TakingPieceOfSameColor method returns true if a piece of the other color
* is eaten. If a player tries to eat a piece of the same color it returns false.
* In the if statement we pass the EmptySquare method. If the square is empty it
* returns always false.
*/
 boolean TakingPieceOfSameColor (int k1, int r1, int k2, int r2){
   if (EmptySquare (k2,r2)) return false;
   return ( (board[k1][r1] % 2) == (board[k2][r2] % 2)) ? true : false ;
 }


/**
* The isItAttacked method cheks if the King is attacked. If the King is attacked
* from other pieces it returns true. Otherwise it returns false. The two for loops
* are used for checking all the positions on the board. We pass to the first if
* statement two methods: EmptySquare and PieceOwner. To the second if statement we
* pass the SemiLegalMove method.
*/
 public boolean isItAttacked(int player, int k, int r){
        boolean isatt = false;
        int kingpos;
        for(int k1 = 0; k1 < 8; k1++){
            for(int r1 = 0; r1 < 8; r1++){
                if((!EmptySquare(k1,r1)) && (PieceOwner(k1, r1)== player)){
                    if(SemiLegalMove(k1,r1,k,r)){
                        isatt = true;
                    }
                }
            }
        }
        return isatt;
  }


/**
 * The IsSpaceFree1_xy method checks if the executed movement is allowed or not.
 * It goes trough the initial until the final position of the moved piece and
 * returns true if the positions in between are free. This method is only
 * good for the horizontal and vertical movements. In all four cases we have a
 * while and an if statement. In the if statement we pass the EmptySquare method.
 * If both boolean products in the while statement are true, the variable is incremented
 * or decremented and the return value is true.
 */
  public boolean IsSpaceFree1_xy(int k1, int r1, int k2, int r2){  //Check for not jumping
        boolean empty = true;

        if(r1==r2){

            int m = k1 + 1;

            while ((m < k2) && empty){
                if (!EmptySquare(m, r1)){
                    empty = false;

                }m++;
            }
            int n = k1 - 1;
            while ((n > k2) && empty){
                if (!EmptySquare(n, r1)){
                    empty = false;

                }n--;
            }

        }

        else if(k1==k2){
            empty = true;
            int s = r1 + 1;

            while ((s < r2) && empty){
                if (!EmptySquare(k1, s)){
                    empty = false;

                }s++;
            }
            int t = r1 - 1;
            while ((t > r2) && empty){
                if (!EmptySquare(k1, t)){
                    empty = false;

                }t--;
            }

        }
        return empty;
  }



 /**
 * The IsSpaceFree2_xy method checks if the executed movement is allowed or not.
 * It goes trough the initial until the final position of the moved piece and
 * returns true if the positions in between are free. This method is only
 * good for the oblique movements. In all four cases we have a while and two if
 * statement. In the if statement we pass the EmptySquare method. The first if
 * statement checks in which direction the piece has been moved. The second contains
 * the EmptySquare method which determins if empty is true or false. If all boolean
 * products in the while staement are true, the return value is true.
 */
  public boolean IsSpaceFree2_xy(int k1, int r1, int k2, int r2){
        boolean empty = true;
        int a = 0;
        int b = 0;
        a = k1 + 1;
        b = r1 + 1;
        if ((r2 > r1) && (k2 > k1)){
            while ((a < k2) && ( b < r2) && empty){
                if (!EmptySquare(a, b)){
                    empty = false;
                }
                a++;
                b++;
            }
        }
        int m = 0;
        int n = 0;
        m = k1 - 1;
        n = r1 - 1;
        if ((r1 > r2) && (k1 > k2)){
            while ((m > k2) && ( n > r2) && empty){
                if (!EmptySquare(m, n)){
                    empty = false;
                }
                m--;
                n--;
            }
        }
        int e = k1 + 1;
        int f = r1 - 1;
        if ((r1 > r2) && (k2 > k1)){
            while ((e < k2) && ( f > r2) && empty){
                if (!EmptySquare(e, f)){
                    empty = false;
                }
                e++;
                f--;
            }
        }
        int c = k1 - 1;
        int d = r1 + 1;
        if ((r2 > r1) && (k1 > k2)){
            while ((c > k2) && ( d < r2) && empty){
                if (!EmptySquare(c, d)){
                    empty = false;
                }
                c--;
                d++;
            }
        }
        return empty;
 }


//////////////////////////////////////////////////////////////////////////////////////////
/**
This method allows to see if a figure on the board is  attacked  or not.
It uses an if-statement to verify this and the if-statement calls another method
the isItAttacked method to see if the figure is in check.
In this method we can found three variables:
the variable player (refers too the black or white player),
the variable k (k indicates the kolumns),
and the variable r (r indicates the rows).
This three variables are passed through the isItAttacked method too.
*/


 public boolean isItInCheck(int type,int k2, int r2){
   if(isItAttacked(type,k2, r2)){
     return true;
   }else{
     return false;
   }
 }


/**
This method checks if the king is in mate or not and to verify this
it uses an if-statement. The if-statement calls two other methods:
the isItInCheck method and the hasFigLegalMove method. If both methods
are true then this method returns that the king is in mate.
In this method we can found three variables:
the variable player(refers too the black or white player),
the variable k (k indicates the kolumns),
and the variable r (r indicates the rows).
 */
 public boolean isItMate(int player, int k, int r){
   boolean inMate = false;
  log.setText("Check: " + String.valueOf(isItInCheck(player,k,r)) + "\n");
  log.setText("LegalMove: " + String.valueOf(hasFigLegalMove(player,k,r)) + "\n");

  if(isItInCheck(player,k,r)&& (!hasFigLegalMove(player, k, r))){
      inMate = true;
  }
  return inMate;
 }


  /**
 * The hasFigLegalMove method checks if the move of the piece is legal.
 * hasMove, a and b are boolean statements. The two for loops are used for
 * checking all the positions on the board. In the if statement we pass two
 * methods: SemiLegalMove and isItAttaked. If the piece has a semilegal move
 * and it is not attacked, the return value for hasMove is true. Otherwise it
 * returns false.
 */
 public boolean hasFigLegalMove(int p, int k, int r){
        boolean hasMove = false;
        /////////////
        boolean a = true;
        boolean b = true;

        for(int k2 = 0; k2 < 8; k2++){
            for(int r2 = 0; r2 < 8; r2++){
                if(a = SemiLegalMove(k, r, k2, r2)&& !(b = isItAttacked(p,k2,r2))){
                    /////////////////////////////////////
                    log.setText("(" + String.valueOf(r2) + "," + String.valueOf(k2) + ") semi:" + String.valueOf(a) + " att:" + String.valueOf(b) + "\n");
                    hasMove = true;
                }
            }
        }
        return hasMove;
    }



 /**
*this method calls the Move method and passes the relative variable to DoMove(...)
*/
 void DoMove (Move m){
   DoMove(m.startcol,m.startrow,m.endcol,m.endrow);
 }


/**
* this method does the actual move, changes the move-owner, updates the board[][] positions
* and sets the "lastmove display" to the updated value
*
*/
 void DoMove (int k1, int r1, int k2, int r2){
     // First determine if this is a promotion move
//promotion types here
     // update status
   lastmove = new Move (k1,r1, k2, r2);
   lastplayermoved = PieceOwner(k1,r1);
   count(k1,r1,k2,r2);
     // do the move
   board[k2][r2] = board[k1][r1];
   board[k1][r1]=0;
   int rag = board[k2][r2];
   String figure = "no value";
   if(rag == 1){
     figure = "white king";
   }else if(rag == 2){
     figure = "black king";
   }else if(rag == 3){
     figure = "white queen";
   }else if(rag == 4){
     figure = "black queen";
   }else if(rag == 5){
     figure = "white rook";
   }else if(rag == 6){
     figure = "black rook";
   }else if(rag == 7){
     figure = "white knight";
   }else if(rag == 8){
     figure = "black knight";
   }else if(rag == 9){
     figure = "white bishop";
   }else if(rag == 2){
     figure = "black bishop";
   }else if(rag == 11){
     figure = "white pawn";
   }else if(rag == 12){
     figure = "black pawn";
   }
   ta.setText(figure);
 }

/**
* this method is the counter for the 50-moves rule. The varialbe moveNumber is the countindex.
* it's set to zero every time one of the 50-move rules is being voilated
*@param k1 startcolon
*@param r1 start row
*@param k2 destination column
*@param r2 destination row
*/
 public void count(int k1, int r1, int k2, int r2){
        String pla;
        int eatRange = 0;
        if((board[k1][r1] == 12)||(board[k1][r1] == 11)){
            moveNumber = 0;
            if((!EmptySquare(k2, r2))&& (!TakingPieceOfSameColor(k1,r1,k2,r2))){
                eatRange = 1;
            }else{
                eatRange = 0;
            }
        }
        if(eatRange == 1){
            moveNumber = 0;
        }
        moveNumber++;
        log.setText(moveNumber + " ");
        if(moveNumber >= 50){
            if(lastplayermoved == 0){
                pla = "Black";
            }else{
                pla = "white";
            }
            log.setText(pla + " can claim a draw");
        }
    }
}
